//-------------------------------------------------------------------------
// 
// $Revision: 1.1 $
// $Date: 2005/04/27 18:58:19 $
//
//-------------------------------------------------------------------------
// Clock / Reset / Startup Logic - Module
//-------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
// 
//-------------------------------------------------------------------------
// Filename:        clk_rst_startup.v
// 
// Description:
//   This module generates the global clocks and reset signals for the 
//   FPGA as well as a startup circuit to hold reset until the DCMs
//   are locked onto the reference clock.
//   
//-------------------------------------------------------------------------
// Structure:   
// 
//
//-------------------------------------------------------------------------
// Author:      KD
// History:
//
//-------------------------------------------------------------------------
// Naming Conventions:
//      active low signals:                   "*_n"
//      clock signals:                        "clk", "clk_div#", "clk_#x" 
//      reset signals:                        "rst", "rst_n" 
//      generics/parameters:                  "C_*" 
//      user defined types:                   "*_TYPE" 
//      state machine next state:             "*_ns" 
//      state machine current state:          "*_cs" 
//      combinatorial signals:                "*_com" 
//      pipelined or register delay signals:  "*_d#" 
//      counter signals:                      "*cnt*"
//      clock enable signals:                 "*_ce" 
//      internal version of output port       "*_i"
//      device pins:                          "*_pin" 
//      ports:                                - Names begin with Uppercase 
//      processes:                            "*_PROCESS" 
//      component instantiations:             "<ENTITY_>I_<#|FUNC>
//-------------------------------------------------------------------------

`timescale 1ns/1ps

///////////////////////////////////////////////////////////////////////////
// Module Declaration
///////////////////////////////////////////////////////////////////////////

module clk_rst_startup (
  DCM_LOCKED,
  SYS_RST_N,
  SYSACE_CLK,           // I
  REFCLK,               // I
  CLK,                  // O
  CLK90,                // O
  CLK180,               // O
  CLK270,               // O
  CLKRAID,              // O
  CLKCPU,               // O
  CLKOCM,               // O
  CLKDV,                // O
  DDR_clk,              // O
  DDR_clkb,             // O
  DDR_clk1,             // O
  DDR_clk1b,            // O
  DDR_clk2,             // O
  DDR_clk2b,            // O
  gtx_clk,              // O
  DCM_psen,
  DCM_psincdec,
  DCM_psclk,
  DCM_psdone            
);

///////////////////////////////////////////////////////////////////////////
// Port Declarations
///////////////////////////////////////////////////////////////////////////

  input SYS_RST_N;
  input  SYSACE_CLK;
  input  REFCLK;
  output CLK;
  output CLK90;
  output CLK180;
  output CLK270;
  output CLKRAID;
  output CLKCPU;
  output CLKOCM;
  output CLKDV;
  output DDR_clk;
  output DDR_clkb;
  output DDR_clk1;
  output DDR_clk1b;
  output DDR_clk2;
  output DDR_clk2b;
  output gtx_clk;
  output DCM_LOCKED;
  input DCM_psen;
  input DCM_psincdec;
  input DCM_psclk;
  output DCM_psdone;

   
///////////////////////////////////////////////////////////////////////////
// Signal Declarations
///////////////////////////////////////////////////////////////////////////

  wire        all_dcm_locked;
  wire        clk_i;
  wire        clk90_i;
  wire        clk270_i;
  wire        clkcpu_i;
  wire        clkdv_i;
  wire        dcm_rst;
  wire        dcm1_locked;
  wire        dcm2_locked;
  wire  [3:0] por_ff;
  wire        sys_rst;
  wire        sysace_clk_i;
  wire        ddr_out_clk_i;
  wire        ddr_out_clk;
  wire        REFCLK_buf;

  wire        clk2x_i;

///////////////////////////////////////////////////////////////////////////
// Buffer and DCM Instantiation
///////////////////////////////////////////////////////////////////////////

assign sys_rst = ~SYS_RST_N;


// BUFG between IBUFG and DCM as per Answer Record 20529
BUFG REFCLK_bufg (.I(REFCLK), .O(REFCLK_buf));

// Instantiate the DCMs to generate the system clocks
DCM_BASE clk_dcm_inst1 (
  .CLKIN    (REFCLK_buf),  // I
  .CLKFB    (CLK),         // I
  .CLK0     (clk_i),       // O
  .CLK90    (clk90_i),     // O
  .CLK180   (),            // O
  .CLK270   (clk270_i),    // O
  .CLK2X    (clk2x_i),            // O//MD this was the original raid clock
  .CLK2X180 (),            // O
  .CLKDV    (clkdv_i),     // O
  .CLKFX    (clkcpu_i),    // O
  .CLKFX180 (),            // O
  .LOCKED   (dcm1_locked), // O
  .RST      (dcm_rst)      // I
);

  // Set the clock multiplier ratio for the CPU
  defparam clk_dcm_inst1.CLKFX_MULTIPLY = 3;
  defparam clk_dcm_inst1.CLKFX_DIVIDE   = 1;
  defparam clk_dcm_inst1.DLL_FREQUENCY_MODE = "LOW";
  defparam clk_dcm_inst1.DFS_FREQUENCY_MODE = "HIGH";
  defparam clk_dcm_inst1.CLKIN_PERIOD = 10;
  defparam clk_dcm_inst1.DUTY_CYCLE_CORRECTION = "TRUE";
  defparam clk_dcm_inst1.CLKDV_DIVIDE = 2;

  // Global Buffers to drive clock outputs from DCM
  BUFG clk_bufg        (.I(clk_i),        .O(CLK));
  BUFG clk90_bufg      (.I(clk90_i),      .O(CLK90));
  assign CLK180 = ~CLK;
  BUFG clk270_bufg     (.I(clk270_i),     .O(CLK270));
  BUFG CLKCPU_bufg     (.I(clkcpu_i),     .O(CLKCPU));
  BUFG sysace_clk_bufg (.I(SYSACE_CLK),   .O(sysace_clk_i));
  BUFG CLKDV_bufg      (.I(clkdv_i),      .O(CLKDV));
  //MD BUFG CLK2X_bufg      (.I(clk2x_i),      .O(CLKRAID));
  BUFG CLKRAID_bufg      (.I(clk_raid_i),      .O(CLKRAID));

   
  assign CLKOCM = CLK;

///////////////////////////////////////////////////////////////////////////
//RAID CLOCK
///////////////////////////////////////////////////////////////////////////
//this is the new raid clock 150MHz
BUFG clk_dcm3_fx_bufg        (.I(clk_r_i),        .O(CLK_R));

DCM_BASE clk_dcm_inst3 (
  .CLKIN    (CLK),             // I
  .CLKFB    (CLK_R),     // I
  .CLK0     (clk_r_i),   // O
  .CLK90    (),                // O
  .CLK180   (),                // O
  .CLK270   (),                // O
  .CLK2X    (),                // O
  .CLK2X180 (),                // O
  .CLKDV    (),                // O
  .CLKFX    (clk_raid_i),         // O
  .CLKFX180 (),                // O
  .LOCKED   (dcm3_locked),     // O
  .RST      (dcm_rst | ~dcm1_locked) //I
);
  defparam clk_dcm_inst3.DLL_FREQUENCY_MODE  = "LOW";
  defparam clk_dcm_inst3.DFS_FREQUENCY_MODE  = "LOW";
  defparam clk_dcm_inst3.CLKFX_MULTIPLY      = 3;
  defparam clk_dcm_inst3.CLKFX_DIVIDE        = 2;
  defparam clk_dcm_inst3.CLKIN_PERIOD        = 10;



///////////////////////////////////////////////////////////////////////////
// Drive DDR clock
///////////////////////////////////////////////////////////////////////////

BUFG ddr_out_clk_bufg   (.I(ddr_out_clk_i),     .O(ddr_out_clk));

DCM_PS clk_dcm_inst2 (
  .CLKIN    (CLK),             // I
  .CLKFB    (ddr_out_clk),     // I
  .CLK0     (ddr_out_clk_i),   // O
  .CLK90    (),                // O
  .CLK180   (),                // O
  .CLK270   (),                // O
  .CLK2X    (),                // O
  .CLK2X180 (),                // O
  .CLKDV    (),                // O
  .CLKFX    (gtx_clk),         // O
  .CLKFX180 (),                // O
  .LOCKED   (dcm2_locked),     // O
  .PSCLK    (DCM_psclk),       // I
  .PSDONE   (DCM_psdone),      // O
  .PSEN     (DCM_psen),        // I
  .PSINCDEC (DCM_psincdec),    // I
  .RST      (dcm_rst | ~dcm1_locked), // I
  .DO       ()                 // O
);
  defparam clk_dcm_inst2.DLL_FREQUENCY_MODE  = "LOW";
  defparam clk_dcm_inst2.DFS_FREQUENCY_MODE  = "LOW";
  defparam clk_dcm_inst2.CLKFX_MULTIPLY      = 5;
  defparam clk_dcm_inst2.CLKFX_DIVIDE        = 4;
  defparam clk_dcm_inst2.CLKIN_PERIOD        = 10;
  defparam clk_dcm_inst2.CLKOUT_PHASE_SHIFT  = "VARIABLE_POSITIVE";
  defparam clk_dcm_inst2.PHASE_SHIFT         = 55;

ODDR DDR_FF_CLK (
  .Q(DDR_clk),
  .D1(1'b1),
  .D2(1'b0),
  .C(ddr_out_clk),
  .CE(1'b1),
  .R(1'b0),
  .S(1'b0)
);

ODDR DDR_FF_CLKB (
  .Q(DDR_clkb),
  .D1(1'b0),
  .D2(1'b1),
  .C(ddr_out_clk),
  .CE(1'b1),
  .R(1'b0),
  .S(1'b0)
);

ODDR DDR_FF_CLK1 (
  .Q(DDR_clk1),
  .D1(1'b1),
  .D2(1'b0),
  .C(ddr_out_clk),
  .CE(1'b1),
  .R(1'b0),
  .S(1'b0)
);

ODDR DDR_FF_CLK1B (
  .Q(DDR_clk1b),
  .D1(1'b0),
  .D2(1'b1),
  .C(ddr_out_clk),
  .CE(1'b1),
  .R(1'b0),
  .S(1'b0)
);

ODDR DDR_FF_CLK2 (
  .Q(DDR_clk2),
  .D1(1'b1),
  .D2(1'b0),
  .C(ddr_out_clk),
  .CE(1'b1),
  .R(1'b0),
  .S(1'b0)
);

ODDR DDR_FF_CLK2B (
  .Q(DDR_clk2b),
  .D1(1'b0),
  .D2(1'b1),
  .C(ddr_out_clk),
  .CE(1'b1),
  .R(1'b0),
  .S(1'b0)
);

///////////////////////////////////////////////////////////////////////////
// Startup / RST Logic
///////////////////////////////////////////////////////////////////////////

  // Use Free Running (Non-DCM controlled) clock to control logic to
  // generate a power-on reset pulse to the DCMs. This ensures a clean
  // startup reset pulse on the DCMs.
   FDP por_ff3 (.C(sysace_clk_i), .D(1'b0), .Q(por_ff[3]), .PRE(sys_rst));
   FDP por_ff2 (.C(sysace_clk_i), .D(por_ff[3]), .Q(por_ff[2]), .PRE(sys_rst));
   FDP por_ff1 (.C(sysace_clk_i), .D(por_ff[2]), .Q(por_ff[1]), .PRE(sys_rst));
   FDP por_ff0 (.C(sysace_clk_i), .D(por_ff[1]), .Q(por_ff[0]), .PRE(sys_rst));

   defparam por_ff3.INIT = 1'b1;
   defparam por_ff2.INIT = 1'b1;
   defparam por_ff1.INIT = 1'b1;
   defparam por_ff0.INIT = 1'b1;

   assign dcm_rst = por_ff[0];
   assign all_dcm_locked = dcm1_locked & dcm2_locked & dcm3_locked; //MD added dcm3
   assign DCM_LOCKED = all_dcm_locked;
   
endmodule

